@@ -1,17 +1,12 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2021 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 """
 .. _codemods:
@@ -50,6 +45,7 @@ at the cost of additional configuration (adding ``'hypothesis.extra'`` to the
 .. autofunction:: refactor
 """
 
+import functools
 import importlib
 from inspect import Parameter, signature
 from typing import List
@@ -103,7 +99,7 @@ class HypothesisFixComplexMinMagnitude(VisitorBasedCodemodCommand):
     METADATA_DEPENDENCIES = (cst.metadata.QualifiedNameProvider,)
 
     @m.call_if_inside(
-        m.Call(metadata=match_qualname("hypothesis.strategies.complex_numbers"))  # type: ignore
+        m.Call(metadata=match_qualname("hypothesis.strategies.complex_numbers"))
     )
     def leave_Arg(self, original_node, updated_node):
         if m.matches(
@@ -113,6 +109,12 @@ class HypothesisFixComplexMinMagnitude(VisitorBasedCodemodCommand):
         return updated_node
 
 
+@functools.lru_cache()
+def get_fn(import_path):
+    mod, fn = import_path.rsplit(".", 1)
+    return getattr(importlib.import_module(mod), fn)
+
+
 class HypothesisFixPositionalKeywonlyArgs(VisitorBasedCodemodCommand):
     """Fix positional arguments for newly keyword-only parameters, e.g.::
 
@@ -195,10 +197,17 @@ class HypothesisFixPositionalKeywonlyArgs(VisitorBasedCodemodCommand):
         # Get the actual function object so that we can inspect the signature.
         # This does e.g. incur a dependency on Numpy to fix Numpy-dependent code,
         # but having a single source of truth about the signatures is worth it.
-        mod, fn = list(qualnames.intersection(self.kwonly_functions))[0].rsplit(".", 1)
         try:
-            func = getattr(importlib.import_module(mod), fn)
-        except ImportError:
+            params = signature(get_fn(*qualnames)).parameters.values()
+        except ModuleNotFoundError:
+            return updated_node
+
+        # st.floats() has a new allow_subnormal kwonly argument not at the end,
+        # so we do a bit more of a dance here.
+        if qualnames == {"hypothesis.strategies.floats"}:
+            params = [p for p in params if p.name != "allow_subnormal"]
+
+        if len(updated_node.args) > len(params):
             return updated_node
 
         # Create new arg nodes with the newly required keywords
@@ -210,6 +219,6 @@ class HypothesisFixPositionalKeywonlyArgs(VisitorBasedCodemodCommand):
             arg
             if arg.keyword or arg.star or p.kind is not Parameter.KEYWORD_ONLY
             else arg.with_changes(keyword=cst.Name(p.name), equal=assign_nospace)
-            for p, arg in zip(signature(func).parameters.values(), updated_node.args)
+            for p, arg in zip(params, updated_node.args)
         ]
         return updated_node.with_changes(args=newargs)
