@@ -1,26 +1,21 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2021 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 import math
-from copy import deepcopy
 from typing import Any, Mapping, Optional, Sequence, Tuple, Union
 
 import numpy as np
 
 from hypothesis import strategies as st
-from hypothesis.errors import InvalidArgument
+from hypothesis._settings import note_deprecation
+from hypothesis.errors import HypothesisException, InvalidArgument
 from hypothesis.extra._array_helpers import (
     NDIM_MAX,
     BasicIndex,
@@ -68,6 +63,9 @@ __all__ = [
 
 TIME_RESOLUTIONS = tuple("Y  M  D  h  m  s  ms  us  ns  ps  fs  as".split())
 
+# See https://github.com/HypothesisWorks/hypothesis/pull/3394 and linked discussion.
+NP_FIXED_UNICODE = tuple(int(x) for x in np.__version__.split(".")[:2]) >= (1, 19)
+
 
 @defines_strategy(force_reusable_values=True)
 def from_dtype(
@@ -80,6 +78,7 @@ def from_dtype(
     max_value: Union[int, float, None] = None,
     allow_nan: Optional[bool] = None,
     allow_infinity: Optional[bool] = None,
+    allow_subnormal: Optional[bool] = None,
     exclude_min: Optional[bool] = None,
     exclude_max: Optional[bool] = None,
 ) -> st.SearchStrategy[Any]:
@@ -125,12 +124,13 @@ def from_dtype(
         result: st.SearchStrategy[Any] = st.booleans()
     elif dtype.kind == "f":
         result = st.floats(
-            width=8 * dtype.itemsize,
+            width=min(8 * dtype.itemsize, 64),
             **compat_kw(
                 "min_value",
                 "max_value",
                 "allow_nan",
                 "allow_infinity",
+                "allow_subnormal",
                 "exclude_min",
                 "exclude_max",
             ),
@@ -160,6 +160,8 @@ def from_dtype(
     elif dtype.kind == "U":
         # Encoded in UTF-32 (four bytes/codepoint) and null-terminated
         max_size = (dtype.itemsize or 0) // 4 or None
+        if NP_FIXED_UNICODE and "alphabet" not in kwargs:
+            kwargs["alphabet"] = st.characters()
         result = st.text(**compat_kw("alphabet", "min_size", max_size=max_size)).filter(
             lambda b: b[-1:] != "\0"
         )
@@ -171,7 +173,7 @@ def from_dtype(
             # it here because we'd have to guard against equivalents in arrays()
             # regardless and drawing scalars is a valid use-case.
             res = st.sampled_from(TIME_RESOLUTIONS)
-        result = st.builds(dtype.type, st.integers(-(2 ** 63), 2 ** 63 - 1), res)
+        result = st.builds(dtype.type, st.integers(-(2**63), 2**63 - 1), res)
     else:
         raise InvalidArgument(f"No strategy inference for {dtype}")
     return result.map(dtype.type)
@@ -187,9 +189,7 @@ class ArrayStrategy(st.SearchStrategy):
         self.unique = unique
         self._check_elements = dtype.kind not in ("O", "V")
 
-    def set_element(self, data, result, idx, strategy=None):
-        strategy = strategy or self.element_strategy
-        val = data.draw(strategy)
+    def set_element(self, val, result, idx, *, fill=False):
         try:
             result[idx] = val
         except TypeError as err:
@@ -197,7 +197,32 @@ class ArrayStrategy(st.SearchStrategy):
                 f"Could not add element={val!r} of {val.dtype!r} to array of "
                 f"{result.dtype!r} - possible mismatch of time units in dtypes?"
             ) from err
-        if self._check_elements and val != result[idx] and val == val:
+        try:
+            elem_changed = self._check_elements and val != result[idx] and val == val
+        except Exception as err:  # pragma: no cover
+            # This branch only exists to help debug weird behaviour in Numpy,
+            # such as the string problems we had a while back.
+            raise HypothesisException(
+                "Internal error when checking element=%r of %r to array of %r"
+                % (val, val.dtype, result.dtype)
+            ) from err
+        if elem_changed:
+            strategy = self.fill if fill else self.element_strategy
+            if self.dtype.kind == "f":
+                try:
+                    is_subnormal = 0 < abs(val) < np.finfo(self.dtype).tiny
+                except Exception:  # pragma: no cover
+                    # val may be a non-float that does not support the
+                    # operations __lt__ and __abs__
+                    is_subnormal = False  # pragma: no cover
+                if is_subnormal:
+                    raise InvalidArgument(  # pragma: no cover
+                        f"Generated subnormal float {val} from strategy "
+                        f"{strategy} resulted in {result[idx]!r}, probably "
+                        "as a result of NumPy being built with flush-to-zero "
+                        "compiler options. Consider passing "
+                        "allow_subnormal=False."
+                    )
             raise InvalidArgument(
                 "Generated array element %r from %r cannot be represented as "
                 "dtype %r - instead it becomes %r (type %r).  Consider using a more "
@@ -230,28 +255,17 @@ class ArrayStrategy(st.SearchStrategy):
             # generate a fully dense array with a freshly drawn value for each
             # entry.
             if self.unique:
-                seen = set()
-                elements = cu.many(
-                    data,
+                elems = st.lists(
+                    self.element_strategy,
                     min_size=self.array_size,
                     max_size=self.array_size,
-                    average_size=self.array_size,
+                    unique=True,
                 )
-                i = 0
-                while elements.more():
-                    # We assign first because this means we check for
-                    # uniqueness after numpy has converted it to the relevant
-                    # type for us. Because we don't increment the counter on
-                    # a duplicate we will overwrite it on the next draw.
-                    self.set_element(data, result, i)
-                    if result[i] not in seen:
-                        seen.add(result[i])
-                        i += 1
-                    else:
-                        elements.reject()
+                for i, v in enumerate(data.draw(elems)):
+                    self.set_element(v, result, i)
             else:
                 for i in range(len(result)):
-                    self.set_element(data, result, i)
+                    self.set_element(data.draw(self.element_strategy), result, i)
         else:
             # We draw numpy arrays as "sparse with an offset". We draw a
             # collection of index assignments within the array and assign
@@ -267,7 +281,10 @@ class ArrayStrategy(st.SearchStrategy):
                 # sqrt isn't chosen for any particularly principled reason. It
                 # just grows reasonably quickly but sublinearly, and for small
                 # arrays it represents a decent fraction of the array size.
-                average_size=math.sqrt(self.array_size),
+                average_size=min(
+                    0.9 * self.array_size,  # ensure small arrays sometimes use fill
+                    max(10, math.sqrt(self.array_size)),  # ...but *only* sometimes
+                ),
             )
 
             needs_fill = np.full(self.array_size, True)
@@ -278,7 +295,7 @@ class ArrayStrategy(st.SearchStrategy):
                 if not needs_fill[i]:
                     elements.reject()
                     continue
-                self.set_element(data, result, i)
+                self.set_element(data.draw(self.element_strategy), result, i)
                 if self.unique:
                     if result[i] in seen:
                         elements.reject()
@@ -301,7 +318,7 @@ class ArrayStrategy(st.SearchStrategy):
                 one_element = np.zeros(
                     shape=1, dtype=object if unsized_string_dtype else self.dtype
                 )
-                self.set_element(data, one_element, 0, self.fill)
+                self.set_element(data.draw(self.fill), one_element, 0, fill=True)
                 if unsized_string_dtype:
                     one_element = one_element.astype(self.dtype)
                 fill_value = one_element[0]
@@ -324,7 +341,7 @@ class ArrayStrategy(st.SearchStrategy):
             if mismatch.any():
                 raise InvalidArgument(
                     "Array elements %r cannot be represented as dtype %r - instead "
-                    "they becomes %r.  Use a more precise strategy, e.g. without "
+                    "they become %r.  Use a more precise strategy, e.g. without "
                     "trailing null bytes, as this will be an error future versions."
                     % (result[mismatch], self.dtype, out[mismatch])
                 )
@@ -367,7 +384,7 @@ def arrays(
       strategy that generates such values.
     * ``elements`` is a strategy for generating values to put in the array.
       If it is None a suitable value will be inferred based on the dtype,
-      which may give any legal value (including eg ``NaN`` for floats).
+      which may give any legal value (including eg NaN for floats).
       If a mapping, it will be passed as ``**kwargs`` to ``from_dtype()``
     * ``fill`` is a strategy that may be used to generate a single background
       value for the array. If None, a suitable default will be inferred
@@ -379,8 +396,9 @@ def arrays(
       distinct from one another. Note that in this case multiple NaN values
       may still be allowed. If fill is also set, the only valid values for
       it to return are NaN values (anything for which :obj:`numpy:numpy.isnan`
-      returns True. So e.g. for complex numbers (nan+1j) is also a valid fill).
-      Note that if unique is set to True the generated values must be hashable.
+      returns True. So e.g. for complex numbers ``nan+1j`` is also a valid fill).
+      Note that if ``unique`` is set to ``True`` the generated values must be
+      hashable.
 
     Arrays of specified ``dtype`` and ``shape`` are generated for example
     like this:
@@ -388,17 +406,11 @@ def arrays(
     .. code-block:: pycon
 
       >>> import numpy as np
+      >>> from hypothesis import strategies as st
       >>> arrays(np.int8, (2, 3)).example()
       array([[-8,  6,  3],
              [-6,  4,  6]], dtype=int8)
-
-    - See :doc:`What you can generate and how <data>`.
-
-    .. code-block:: pycon
-
-      >>> import numpy as np
-      >>> from hypothesis.strategies import floats
-      >>> arrays(np.float, 3, elements=floats(0, 1)).example()
+      >>> arrays(np.float, 3, elements=st.floats(0, 1)).example()
       array([ 0.88974794,  0.77387938,  0.1977879 ])
 
     Array values are generated in two parts:
@@ -406,14 +418,14 @@ def arrays(
     1. Some subset of the coordinates of the array are populated with a value
        drawn from the elements strategy (or its inferred form).
     2. If any coordinates were not assigned in the previous step, a single
-       value is drawn from the fill strategy and is assigned to all remaining
+       value is drawn from the ``fill`` strategy and is assigned to all remaining
        places.
 
-    You can set fill to :func:`~hypothesis.strategies.nothing` if you want to
+    You can set :func:`fill=nothing() <hypothesis.strategies.nothing>` to
     disable this behaviour and draw a value for every element.
 
-    If fill is set to None then it will attempt to infer the correct behaviour
-    automatically: If unique is True, no filling will occur by default.
+    If ``fill=None``, then it will attempt to infer the correct behaviour
+    automatically. If ``unique`` is ``True``, no filling will occur by default.
     Otherwise, if it looks safe to reuse the values of elements across
     multiple coordinates (this will be the case for any inferred strategy, and
     for most of the builtins, but is not the case for mutable values or
@@ -734,7 +746,11 @@ def nested_dtypes(
     ).filter(lambda d: max_itemsize is None or d.itemsize <= max_itemsize)
 
 
-valid_tuple_axes = deepcopy(_valid_tuple_axes)
+@proxies(_valid_tuple_axes)
+def valid_tuple_axes(*args, **kwargs):
+    return _valid_tuple_axes(*args, **kwargs)
+
+
 valid_tuple_axes.__doc__ = f"""
     Return a strategy for generating permissible tuple-values for the
     ``axis`` argument for a numpy sequential function (e.g.
@@ -745,18 +761,23 @@ valid_tuple_axes.__doc__ = f"""
     """
 
 
-mutually_broadcastable_shapes = deepcopy(_mutually_broadcastable_shapes)
+@proxies(_mutually_broadcastable_shapes)
+def mutually_broadcastable_shapes(*args, **kwargs):
+    return _mutually_broadcastable_shapes(*args, **kwargs)
+
+
 mutually_broadcastable_shapes.__doc__ = f"""
     {_mutually_broadcastable_shapes.__doc__}
 
     **Use with Generalised Universal Function signatures**
 
-    A :np-ref:`universal function <ufuncs.html>` (or ufunc for short) is a function
+    A :doc:`universal function <numpy:reference/ufuncs>` (or ufunc for short) is a function
     that operates on ndarrays in an element-by-element fashion, supporting array
     broadcasting, type casting, and several other standard features.
-    A :np-ref:`generalised ufunc <c-api/generalized-ufuncs.html>` operates on
+    A :doc:`generalised ufunc <numpy:reference/c-api/generalized-ufuncs>` operates on
     sub-arrays rather than elements, based on the "signature" of the function.
-    Compare e.g. :obj:`numpy:numpy.add` (ufunc) to :obj:`numpy:numpy.matmul` (gufunc).
+    Compare e.g. :obj:`numpy.add() <numpy:numpy.add>` (ufunc) to
+    :obj:`numpy.matmul() <numpy:numpy.matmul>` (gufunc).
 
     To generate shapes for a gufunc, you can pass the ``signature`` argument instead of
     ``num_shapes``.  This must be a gufunc signature string; which you can write by
@@ -795,7 +816,7 @@ def basic_indices(
     allow_newaxis: bool = False,
     allow_ellipsis: bool = True,
 ) -> st.SearchStrategy[BasicIndex]:
-    """Return a strategy for :np-ref:`basic indexes <arrays.indexing.html>` of
+    """Return a strategy for :doc:`basic indexes <numpy:reference/arrays.indexing>` of
     arrays with the specified shape, which may include dimensions of size zero.
 
     It generates tuples containing some mix of integers, :obj:`python:slice`
@@ -808,10 +829,11 @@ def basic_indices(
       a valid index; for one-dimensional arrays use
       :func:`~hypothesis.strategies.slices` instead.
     * ``min_dims`` is the minimum dimensionality of the resulting array from use
-      of the generated index. When ``min_dims == 0``, indices for zero-dimensional
-      arrays are generated.
+      of the generated index. When ``min_dims == 0``, scalars and zero-dimensional
+      arrays are both allowed.
     * ``max_dims`` is the the maximum dimensionality of the resulting array,
-      defaulting to ``max(len(shape), min_dims) + 2``.
+      defaulting to ``len(shape) if not allow_newaxis else
+      max(len(shape), min_dims) + 2``.
     * ``allow_newaxis`` specifies whether ``None`` is allowed in the index.
     * ``allow_ellipsis`` specifies whether ``...`` is allowed in the index.
     """
@@ -820,29 +842,49 @@ def basic_indices(
     # cases if they're dealing in general indexers, and while it's fiddly we can
     # back-compatibly add them later (hence using kwonlyargs).
     check_type(tuple, shape, "shape")
+    check_argument(
+        all(isinstance(x, int) and x >= 0 for x in shape),
+        f"shape={shape!r}, but all dimensions must be non-negative integers.",
+    )
     check_type(bool, allow_ellipsis, "allow_ellipsis")
     check_type(bool, allow_newaxis, "allow_newaxis")
     check_type(int, min_dims, "min_dims")
+    if min_dims > len(shape) and not allow_newaxis:
+        note_deprecation(
+            f"min_dims={min_dims} is larger than len(shape)={len(shape)}, "
+            "but allow_newaxis=False makes it impossible for an indexing "
+            "operation to add dimensions.",
+            since="2021-09-15",
+            has_codemod=False,
+        )
     check_valid_dims(min_dims, "min_dims")
 
     if max_dims is None:
-        max_dims = min(max(len(shape), min_dims) + 2, NDIM_MAX)
-    check_type(int, max_dims, "max_dims")
+        if allow_newaxis:
+            max_dims = min(max(len(shape), min_dims) + 2, NDIM_MAX)
+        else:
+            max_dims = min(len(shape), NDIM_MAX)
+    else:
+        check_type(int, max_dims, "max_dims")
+        if max_dims > len(shape) and not allow_newaxis:
+            note_deprecation(
+                f"max_dims={max_dims} is larger than len(shape)={len(shape)}, "
+                "but allow_newaxis=False makes it impossible for an indexing "
+                "operation to add dimensions.",
+                since="2021-09-15",
+                has_codemod=False,
+            )
     check_valid_dims(max_dims, "max_dims")
 
     order_check("dims", 0, min_dims, max_dims)
 
-    if not all(isinstance(x, int) and x >= 0 for x in shape):
-        raise InvalidArgument(
-            f"shape={shape!r}, but all dimensions must be of integer size >= 0"
-        )
-
     return BasicIndexStrategy(
         shape,
         min_dims=min_dims,
         max_dims=max_dims,
         allow_ellipsis=allow_ellipsis,
         allow_newaxis=allow_newaxis,
+        allow_fewer_indices_than_dims=True,
     )
 
 
