@@ -1,20 +1,16 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2021 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 from collections import OrderedDict, abc
 from copy import copy
+from datetime import datetime, timedelta
 from typing import Any, List, Optional, Sequence, Set, Union
 
 import attr
@@ -22,11 +18,13 @@ import numpy as np
 import pandas
 
 from hypothesis import strategies as st
+from hypothesis._settings import note_deprecation
 from hypothesis.control import reject
 from hypothesis.errors import InvalidArgument
 from hypothesis.extra import numpy as npst
 from hypothesis.internal.conjecture import utils as cu
 from hypothesis.internal.coverage import check, check_function
+from hypothesis.internal.reflection import get_pretty_function_description
 from hypothesis.internal.validation import (
     check_type,
     check_valid_interval,
@@ -82,6 +80,19 @@ def elements_and_dtype(elements, dtype, source=None):
                 f"{prefix}dtype is categorical, which is currently unsupported"
             )
 
+    if isinstance(dtype, type) and np.dtype(dtype).kind == "O" and dtype is not object:
+        note_deprecation(
+            f"Passed dtype={dtype!r} is not a valid Pandas dtype.  We'll treat it as "
+            "dtype=object for now, but this will be an error in a future version.",
+            since="2021-12-31",
+            has_codemod=False,
+        )
+
+    if isinstance(dtype, st.SearchStrategy):
+        raise InvalidArgument(
+            f"Passed dtype={dtype!r} is a strategy, but we require a concrete dtype "
+            "here.  See https://stackoverflow.com/q/74355937 for workaround patterns."
+        )
     dtype = try_convert(np.dtype, dtype, "dtype")
 
     if elements is None:
@@ -96,11 +107,11 @@ def elements_and_dtype(elements, dtype, source=None):
                 raise InvalidArgument(
                     "Cannot convert %s=%r of type %s to dtype %s"
                     % (name, value, type(value).__name__, dtype.str)
-                )
+                ) from None
             except ValueError:
                 raise InvalidArgument(
                     f"Cannot convert {name}={value!r} to type {dtype.str}"
-                )
+                ) from None
 
         elements = elements.map(convert_element)
     assert elements is not None
@@ -109,13 +120,14 @@ def elements_and_dtype(elements, dtype, source=None):
 
 
 class ValueIndexStrategy(st.SearchStrategy):
-    def __init__(self, elements, dtype, min_size, max_size, unique):
+    def __init__(self, elements, dtype, min_size, max_size, unique, name):
         super().__init__()
         self.elements = elements
         self.dtype = dtype
         self.min_size = min_size
         self.max_size = max_size
         self.unique = unique
+        self.name = name
 
     def do_draw(self, data):
         result = []
@@ -141,7 +153,9 @@ class ValueIndexStrategy(st.SearchStrategy):
         dtype = infer_dtype_if_necessary(
             dtype=self.dtype, values=result, elements=self.elements, draw=data.draw
         )
-        return pandas.Index(result, dtype=dtype, tupleize_cols=False)
+        return pandas.Index(
+            result, dtype=dtype, tupleize_cols=False, name=data.draw(self.name)
+        )
 
 
 DEFAULT_MAX_SIZE = 10
@@ -165,7 +179,7 @@ def range_indexes(
     check_valid_size(min_size, "min_size")
     check_valid_size(max_size, "max_size")
     if max_size is None:
-        max_size = min([min_size + DEFAULT_MAX_SIZE, 2 ** 63 - 1])
+        max_size = min([min_size + DEFAULT_MAX_SIZE, 2**63 - 1])
     check_valid_interval(min_size, max_size, "min_size", "max_size")
     return st.integers(min_size, max_size).map(pandas.RangeIndex)
 
@@ -179,6 +193,7 @@ def indexes(
     min_size: int = 0,
     max_size: Optional[int] = None,
     unique: bool = True,
+    name: st.SearchStrategy[Optional[str]] = st.none(),
 ) -> st.SearchStrategy[pandas.Index]:
     """Provides a strategy for producing a :class:`pandas.Index`.
 
@@ -198,6 +213,8 @@ def indexes(
       should pass a max_size explicitly.
     * unique specifies whether all of the elements in the resulting index
       should be distinct.
+    * name is a strategy for strings or ``None``, which will be passed to
+      the :class:`pandas.Index` constructor.
     """
     check_valid_size(min_size, "min_size")
     check_valid_size(max_size, "max_size")
@@ -208,7 +225,7 @@ def indexes(
 
     if max_size is None:
         max_size = min_size + DEFAULT_MAX_SIZE
-    return ValueIndexStrategy(elements, dtype, min_size, max_size, unique)
+    return ValueIndexStrategy(elements, dtype, min_size, max_size, unique, name)
 
 
 @defines_strategy()
@@ -219,6 +236,7 @@ def series(
     index: Optional[st.SearchStrategy[Union[Sequence, pandas.Index]]] = None,
     fill: Optional[st.SearchStrategy[Ex]] = None,
     unique: bool = False,
+    name: st.SearchStrategy[Optional[str]] = st.none(),
 ) -> st.SearchStrategy[pandas.Series]:
     """Provides a strategy for producing a :class:`pandas.Series`.
 
@@ -245,6 +263,9 @@ def series(
       :func:`~hypothesis.extra.pandas.range_indexes` function to produce
       values for this argument.
 
+    * name: is a strategy for strings or ``None``, which will be passed to
+      the :class:`pandas.Series` constructor.
+
     Usage:
 
     .. code-block:: pycon
@@ -289,7 +310,7 @@ def series(
                     )
                 )
 
-            return pandas.Series(result_data, index=index, dtype=dtype)
+            return pandas.Series(result_data, index=index, dtype=dtype, name=draw(name))
         else:
             return pandas.Series(
                 (),
@@ -297,6 +318,7 @@ def series(
                 dtype=dtype
                 if dtype is not None
                 else draw(dtype_for_elements_strategy(elements)),
+                name=draw(name),
             )
 
     return result()
@@ -322,7 +344,7 @@ class column:
 
     name = attr.ib(default=None)
     elements = attr.ib(default=None)
-    dtype = attr.ib(default=None)
+    dtype = attr.ib(default=None, repr=get_pretty_function_description)
     fill = attr.ib(default=None)
     unique = attr.ib(default=False)
 
@@ -515,10 +537,9 @@ def data_frames(
                 hash(c.name)
             except TypeError:
                 raise InvalidArgument(
-                    "Column names must be hashable, but columns[%d].name was "
-                    "%r of type %s, which cannot be hashed."
-                    % (i, c.name, type(c.name).__name__)
-                )
+                    f"Column names must be hashable, but columns[{i}].name was "
+                    f"{c.name!r} of type {type(c.name).__name__}, which cannot be hashed."
+                ) from None
 
         if c.name in column_names:
             raise InvalidArgument(f"duplicate definition of column name {c.name!r}")
@@ -529,8 +550,7 @@ def data_frames(
 
         if c.dtype is None and rows is not None:
             raise InvalidArgument(
-                "Must specify a dtype for all columns when combining rows with"
-                " columns."
+                "Must specify a dtype for all columns when combining rows with columns."
             )
 
         c.fill = npst.fill_for(
@@ -582,7 +602,21 @@ def data_frames(
                                 reject()
                         else:
                             value = draw(c.elements)
-                        data[c.name][i] = value
+                        try:
+                            data[c.name][i] = value
+                        except ValueError as err:  # pragma: no cover
+                            # This just works in Pandas 1.4 and later, but gives
+                            # a confusing error on previous versions.
+                            if c.dtype is None and not isinstance(
+                                value, (float, int, str, bool, datetime, timedelta)
+                            ):
+                                raise ValueError(
+                                    f"Failed to add value={value!r} to column "
+                                    f"{c.name} with dtype=None.  Maybe passing "
+                                    "dtype=object would help?"
+                                ) from err
+                            # Unclear how this could happen, but users find a way...
+                            raise
 
             for c in rewritten_columns:
                 if not c.fill.is_empty:
@@ -669,11 +703,8 @@ def data_frames(
 
                     if len(row) > len(rewritten_columns):
                         raise InvalidArgument(
-                            (
-                                "Row %r contains too many entries. Has %d but "
-                                "expected at most %d"
-                            )
-                            % (original_row, len(row), len(rewritten_columns))
+                            f"Row {original_row!r} contains too many entries. Has "
+                            f"{len(row)} but expected at most {len(rewritten_columns)}"
                         )
                     while len(row) < len(rewritten_columns):
                         c = rewritten_columns[len(row)]
