@@ -1,66 +1,61 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2021 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 import codecs
-import importlib
 import inspect
 import platform
 import sys
 import typing
 
+try:
+    BaseExceptionGroup = BaseExceptionGroup
+    ExceptionGroup = ExceptionGroup  # pragma: no cover
+except NameError:
+    from exceptiongroup import (
+        BaseExceptionGroup as BaseExceptionGroup,
+        ExceptionGroup as ExceptionGroup,
+    )
+if typing.TYPE_CHECKING:  # pragma: no cover
+    from typing_extensions import Concatenate as Concatenate, ParamSpec as ParamSpec
+else:
+    try:
+        from typing import Concatenate as Concatenate, ParamSpec as ParamSpec
+    except ImportError:
+        try:
+            from typing_extensions import (
+                Concatenate as Concatenate,
+                ParamSpec as ParamSpec,
+            )
+        except ImportError:
+            Concatenate, ParamSpec = None, None
+
 PYPY = platform.python_implementation() == "PyPy"
 WINDOWS = platform.system() == "Windows"
 
 
-def escape_unicode_characters(s):
+def escape_unicode_characters(s: str) -> str:
     return codecs.encode(s, "unicode_escape").decode("ascii")
 
 
-def int_from_bytes(data):
+def int_from_bytes(data: typing.Union[bytes, bytearray]) -> int:
     return int.from_bytes(data, "big")
 
 
-def int_to_bytes(i, size):
+def int_to_bytes(i: int, size: int) -> bytes:
     return i.to_bytes(size, "big")
 
 
-def int_to_byte(i):
+def int_to_byte(i: int) -> bytes:
     return bytes([i])
 
 
-def qualname(f):
-    try:
-        return f.__qualname__
-    except AttributeError:
-        return f.__name__
-
-
-try:
-    # These types are new in Python 3.7, but also (partially) backported to the
-    # typing backport on PyPI.  Use if possible; or fall back to older names.
-    typing_root_type = (typing._Final, typing._GenericAlias)  # type: ignore
-    ForwardRef = typing.ForwardRef  # type: ignore
-except AttributeError:
-    typing_root_type = (typing.TypingMeta, typing.TypeVar)  # type: ignore
-    try:
-        typing_root_type += (typing._Union,)  # type: ignore
-    except AttributeError:
-        pass
-    ForwardRef = typing._ForwardRef  # type: ignore
-
-
 def is_typed_named_tuple(cls):
     """Return True if cls is probably a subtype of `typing.NamedTuple`.
 
@@ -89,46 +84,43 @@ def get_type_hints(thing):
     Never errors: instead of raising TypeError for uninspectable objects, or
     NameError for unresolvable forward references, just return an empty dict.
     """
+    kwargs = {} if sys.version_info[:2] < (3, 9) else {"include_extras": True}
+
     try:
-        hints = typing.get_type_hints(thing)
-    except (AttributeError, TypeError, NameError):
+        hints = typing.get_type_hints(thing, **kwargs)
+    except (AttributeError, TypeError, NameError):  # pragma: no cover
         hints = {}
 
-    if not inspect.isclass(thing):
-        return hints
-
-    try:
-        hints.update(typing.get_type_hints(thing.__init__))
-    except (TypeError, NameError, AttributeError):
-        pass
+    if inspect.isclass(thing):
+        try:
+            hints.update(typing.get_type_hints(thing.__init__, **kwargs))
+        except (TypeError, NameError, AttributeError):
+            pass
 
     try:
         if hasattr(thing, "__signature__"):
             # It is possible for the signature and annotations attributes to
             # differ on an object due to renamed arguments.
-            # To prevent missing arguments we use the signature to provide any type
-            # hints it has and then override any common names with the more
-            # comprehensive type information from get_type_hints
-            # See https://github.com/HypothesisWorks/hypothesis/pull/2580
-            # for more details.
+            from hypothesis.internal.reflection import get_signature
             from hypothesis.strategies._internal.types import is_a_type
 
             vkinds = (inspect.Parameter.VAR_POSITIONAL, inspect.Parameter.VAR_KEYWORD)
-            for p in inspect.signature(thing).parameters.values():
-                if p.kind not in vkinds and is_a_type(p.annotation):
+            for p in get_signature(thing).parameters.values():
+                if (
+                    p.kind not in vkinds
+                    and is_a_type(p.annotation)
+                    and p.annotation is not p.empty
+                ):
                     if p.default is None:
                         hints[p.name] = typing.Optional[p.annotation]
                     else:
                         hints[p.name] = p.annotation
-    except (AttributeError, TypeError, NameError):
+    except (AttributeError, TypeError, NameError):  # pragma: no cover
         pass
 
     return hints
 
 
-importlib_invalidate_caches = getattr(importlib, "invalidate_caches", lambda: ())
-
-
 def update_code_location(code, newfile, newlineno):
     """Take a code object and lie shamelessly about where it comes from.
 
@@ -145,30 +137,31 @@ def update_code_location(code, newfile, newlineno):
         # added to facilitate future-proof code.  See BPO-37032 for details.
         return code.replace(co_filename=newfile, co_firstlineno=newlineno)
 
-    # This field order is accurate for 3.5 - 3.7, but not 3.8 when a new field
-    # was added for positional-only arguments.  However it also added a .replace()
-    # method that we use instead of field indices, so they're fine as-is.
-    CODE_FIELD_ORDER = [
-        "co_argcount",
-        "co_kwonlyargcount",
-        "co_nlocals",
-        "co_stacksize",
-        "co_flags",
-        "co_code",
-        "co_consts",
-        "co_names",
-        "co_varnames",
-        "co_filename",
-        "co_name",
-        "co_firstlineno",
-        "co_lnotab",
-        "co_freevars",
-        "co_cellvars",
-    ]
-    unpacked = [getattr(code, name) for name in CODE_FIELD_ORDER]
-    unpacked[CODE_FIELD_ORDER.index("co_filename")] = newfile
-    unpacked[CODE_FIELD_ORDER.index("co_firstlineno")] = newlineno
-    return type(code)(*unpacked)
+    else:  # pragma: no cover
+        # This field order is accurate for 3.5 - 3.7, but not 3.8 when a new field
+        # was added for positional-only arguments.  However it also added a .replace()
+        # method that we use instead of field indices, so they're fine as-is.
+        CODE_FIELD_ORDER = [
+            "co_argcount",
+            "co_kwonlyargcount",
+            "co_nlocals",
+            "co_stacksize",
+            "co_flags",
+            "co_code",
+            "co_consts",
+            "co_names",
+            "co_varnames",
+            "co_filename",
+            "co_name",
+            "co_firstlineno",
+            "co_lnotab",
+            "co_freevars",
+            "co_cellvars",
+        ]
+        unpacked = [getattr(code, name) for name in CODE_FIELD_ORDER]
+        unpacked[CODE_FIELD_ORDER.index("co_filename")] = newfile
+        unpacked[CODE_FIELD_ORDER.index("co_firstlineno")] = newlineno
+        return type(code)(*unpacked)
 
 
 # Under Python 2, math.floor and math.ceil returned floats, which cannot
@@ -194,9 +187,10 @@ def ceil(x):
 def bad_django_TestCase(runner):
     if runner is None or "django.test" not in sys.modules:
         return False
-    if not isinstance(runner, sys.modules["django.test"].TransactionTestCase):
-        return False
+    else:  # pragma: no cover
+        if not isinstance(runner, sys.modules["django.test"].TransactionTestCase):
+            return False
 
-    from hypothesis.extra.django._impl import HypothesisTestCase
+        from hypothesis.extra.django._impl import HypothesisTestCase
 
-    return not isinstance(runner, HypothesisTestCase)
+        return not isinstance(runner, HypothesisTestCase)
