@@ -1,17 +1,12 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2021 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 from collections import defaultdict
 from typing import Dict
@@ -101,8 +96,8 @@ def defines_shrink_pass():
     def accept(run_step):
         ShrinkPassDefinition(run_with_chooser=run_step)
 
-        def run(self):  # pragma: no cover
-            raise AssertionError("Shrink passes should not be run directly")
+        def run(self):
+            raise NotImplementedError("Shrink passes should not be run directly")
 
         run.__name__ = run_step.__name__
         run.is_shrink_pass = True
@@ -246,7 +241,7 @@ class Shrinker:
 
     """
 
-    def derived_value(fn):
+    def derived_value(fn):  # noqa: B902
         """It's useful during shrinking to have access to derived values of
         the current shrink target.
 
@@ -283,8 +278,8 @@ class Shrinker:
 
         # We keep track of the current best example on the shrink_target
         # attribute.
-        self.shrink_target = None
-        self.update_shrink_target(initial)
+        self.shrink_target = initial
+        self.clear_change_tracking()
         self.shrinks = 0
 
         # We terminate shrinks that seem to have reached their logical
@@ -452,25 +447,15 @@ class Shrinker:
                     return "s" if n != 1 else ""
 
                 total_deleted = self.initial_size - len(self.shrink_target.buffer)
-
-                self.debug("---------------------")
-                self.debug("Shrink pass profiling")
-                self.debug("---------------------")
-                self.debug("")
                 calls = self.engine.call_count - self.initial_calls
+
                 self.debug(
-                    (
-                        "Shrinking made a total of %d call%s "
-                        "of which %d shrank. This deleted %d byte%s out of %d."
-                    )
-                    % (
-                        calls,
-                        s(calls),
-                        self.shrinks,
-                        total_deleted,
-                        s(total_deleted),
-                        self.initial_size,
-                    )
+                    "---------------------\n"
+                    "Shrink pass profiling\n"
+                    "---------------------\n\n"
+                    f"Shrinking made a total of {calls} call{s(calls)} of which "
+                    f"{self.shrinks} shrank. This deleted {total_deleted} bytes out "
+                    f"of {self.initial_size}."
                 )
                 for useful in [True, False]:
                     self.debug("")
@@ -488,10 +473,8 @@ class Shrinker:
                             continue
 
                         self.debug(
-                            (
-                                "  * %s made %d call%s of which "
-                                "%d shrank, deleting %d byte%s."
-                            )
+                            "  * %s made %d call%s of which "
+                            "%d shrank, deleting %d byte%s."
                             % (
                                 p.name,
                                 p.calls,
@@ -837,22 +820,17 @@ class Shrinker:
 
     def update_shrink_target(self, new_target):
         assert isinstance(new_target, ConjectureResult)
-        if self.shrink_target is not None:
-            self.shrinks += 1
-            # If we are just taking a long time to shrink we don't want to
-            # trigger this heuristic, so whenever we shrink successfully
-            # we give ourselves a bit of breathing room to make sure we
-            # would find a shrink that took that long to find the next time.
-            # The case where we're taking a long time but making steady
-            # progress is handled by `finish_shrinking_deadline` in engine.py
-            self.max_stall = max(
-                self.max_stall, (self.calls - self.calls_at_last_shrink) * 2
-            )
-            self.calls_at_last_shrink = self.calls
-        else:
-            self.__all_changed_blocks = set()
-            self.__last_checked_changed_at = new_target
-
+        self.shrinks += 1
+        # If we are just taking a long time to shrink we don't want to
+        # trigger this heuristic, so whenever we shrink successfully
+        # we give ourselves a bit of breathing room to make sure we
+        # would find a shrink that took that long to find the next time.
+        # The case where we're taking a long time but making steady
+        # progress is handled by `finish_shrinking_deadline` in engine.py
+        self.max_stall = max(
+            self.max_stall, (self.calls - self.calls_at_last_shrink) * 2
+        )
+        self.calls_at_last_shrink = self.calls
         self.shrink_target = new_target
         self.__derived_values = {}
 
@@ -1081,12 +1059,12 @@ class Shrinker:
             lambda ex: (
                 ex.label == DRAW_FLOAT_LABEL
                 and len(ex.children) == 2
-                and ex.children[0].length == 8
+                and ex.children[1].length == 8
             ),
         )
 
-        u = ex.children[0].start
-        v = ex.children[0].end
+        u = ex.children[1].start
+        v = ex.children[1].end
         buf = self.shrink_target.buffer
         b = buf[u:v]
         f = lex_to_float(int_from_bytes(b))
@@ -1327,8 +1305,8 @@ class Shrinker:
                         attempt[u:v] = int_to_bytes(value - 1, v - u)
                 elif d == "X":
                     del attempt[u:v]
-                else:  # pragma: no cover
-                    raise AssertionError(f"Unrecognised command {d!r}")
+                else:
+                    raise NotImplementedError(f"Unrecognised command {d!r}")
         return self.incorporate_new_buffer(attempt)
 
 
