@@ -1,17 +1,12 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2021 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 """This module provides support for a stateful style of testing, where tests
 attempt to find a sequence of operations that cause a breakage rather than just
@@ -25,7 +20,6 @@ import inspect
 from copy import copy
 from functools import lru_cache
 from io import StringIO
-from operator import attrgetter
 from typing import (
     Any,
     Callable,
@@ -42,12 +36,23 @@ from unittest import TestCase
 import attr
 
 from hypothesis import strategies as st
-from hypothesis._settings import HealthCheck, Verbosity, settings as Settings
+from hypothesis._settings import (
+    HealthCheck,
+    Verbosity,
+    note_deprecation,
+    settings as Settings,
+)
 from hypothesis.control import current_build_context
 from hypothesis.core import TestFunc, given
 from hypothesis.errors import InvalidArgument, InvalidDefinition
 from hypothesis.internal.conjecture import utils as cu
-from hypothesis.internal.reflection import function_digest, nicerepr, proxies
+from hypothesis.internal.healthcheck import fail_health_check
+from hypothesis.internal.reflection import (
+    function_digest,
+    get_pretty_function_description,
+    nicerepr,
+    proxies,
+)
 from hypothesis.internal.validation import check_type
 from hypothesis.reporting import current_verbosity, report
 from hypothesis.strategies._internal.featureflags import FeatureStrategy
@@ -112,7 +117,7 @@ def run_state_machine_as_test(state_machine_factory, *, settings=None):
         try:
             if print_steps:
                 report(f"state = {machine.__class__.__name__}()")
-            machine.check_invariants()
+            machine.check_invariants(settings)
             max_steps = settings.stateful_step_count
             steps_run = 0
 
@@ -181,13 +186,20 @@ def run_state_machine_as_test(state_machine_factory, *, settings=None):
                                 )
                         else:
                             machine._add_result_to_targets(rule.targets, result)
+                    elif result is not None:
+                        fail_health_check(
+                            settings,
+                            "Rules should return None if they have no target bundle, "
+                            f"but {rule.function.__qualname__} returned {result!r}",
+                            HealthCheck.return_value,
+                        )
                 finally:
                     if print_steps:
                         # 'result' is only used if the step has target bundles.
                         # If it does, and the result is a 'MultipleResult',
                         # then 'print_step' prints a multi-variable assignment.
                         machine._print_step(rule, data_to_print, result)
-                machine.check_invariants()
+                machine.check_invariants(settings)
                 cd.stop_example()
         finally:
             if print_steps:
@@ -211,16 +223,16 @@ def run_state_machine_as_test(state_machine_factory, *, settings=None):
 
 
 class StateMachineMeta(type):
-    def __setattr__(self, name, value):
+    def __setattr__(cls, name, value):
         if name == "settings" and isinstance(value, Settings):
             raise AttributeError(
                 (
                     "Assigning {cls}.settings = {value} does nothing. Assign "
                     "to {cls}.TestCase.settings, or use @{value} as a decorator "
                     "on the {cls} class."
-                ).format(cls=self.__name__, value=value)
+                ).format(cls=cls.__name__, value=value)
             )
-        return type.__setattr__(self, name, value)
+        return super().__setattr__(name, value)
 
 
 class RuleBasedStateMachine(metaclass=StateMachineMeta):
@@ -237,7 +249,7 @@ class RuleBasedStateMachine(metaclass=StateMachineMeta):
     _invariants_per_class: Dict[type, List[classmethod]] = {}
     _initializers_per_class: Dict[type, List[classmethod]] = {}
 
-    def __init__(self):
+    def __init__(self) -> None:
         if not self.rules():
             raise InvalidDefinition(f"Type {type(self).__name__} defines no rules")
         self.bundles: Dict[str, list] = {}
@@ -321,16 +333,16 @@ class RuleBasedStateMachine(metaclass=StateMachineMeta):
 
     def _print_step(self, rule, data, result):
         self.step_count = getattr(self, "step_count", 0) + 1
-        # If the step has target bundles, and the result is a MultipleResults
-        # then we want to assign to multiple variables.
-        if isinstance(result, MultipleResults):
-            n_output_vars = len(result.values)
-        else:
-            n_output_vars = 1
-        if rule.targets and n_output_vars >= 1:
-            output_assignment = ", ".join(self._last_names(n_output_vars)) + " = "
-        else:
-            output_assignment = ""
+        output_assignment = ""
+        if rule.targets:
+            if isinstance(result, MultipleResults):
+                if len(result.values) == 1:
+                    output_assignment = f"({self._last_names(1)[0]},) = "
+                elif result.values:
+                    output_names = self._last_names(len(result.values))
+                    output_assignment = ", ".join(output_names) + " = "
+            else:
+                output_assignment = self._last_names(1)[0] + " = "
         report(
             "{}state.{}({})".format(
                 output_assignment,
@@ -348,13 +360,26 @@ class RuleBasedStateMachine(metaclass=StateMachineMeta):
         for target in targets:
             self.bundles.setdefault(target, []).append(VarReference(name))
 
-    def check_invariants(self):
+    def check_invariants(self, settings):
         for invar in self.invariants():
             if self._initialize_rules_to_run and not invar.check_during_init:
                 continue
             if not all(precond(self) for precond in invar.preconditions):
                 continue
-            invar.function(self)
+            if (
+                current_build_context().is_final
+                or settings.verbosity >= Verbosity.debug
+            ):
+                report(f"state.{invar.function.__name__}()")
+            result = invar.function(self)
+            if result is not None:
+                fail_health_check(
+                    settings,
+                    "The return value of an @invariant is always ignored, but "
+                    f"{invar.function.__qualname__} returned {result!r} "
+                    "instead of None",
+                    HealthCheck.return_value,
+                )
 
     def teardown(self):
         """Called after a run has finished executing to clean up any necessary
@@ -367,26 +392,24 @@ class RuleBasedStateMachine(metaclass=StateMachineMeta):
 
     @classmethod
     @lru_cache()
-    def _to_test_case(state_machine_class):
+    def _to_test_case(cls):
         class StateMachineTestCase(TestCase):
             settings = Settings(deadline=None, suppress_health_check=HealthCheck.all())
 
             def runTest(self):
-                run_state_machine_as_test(state_machine_class)
+                run_state_machine_as_test(cls)
 
             runTest.is_hypothesis_test = True
 
-        StateMachineTestCase.__name__ = state_machine_class.__name__ + ".TestCase"
-        StateMachineTestCase.__qualname__ = (
-            state_machine_class.__qualname__ + ".TestCase"
-        )
+        StateMachineTestCase.__name__ = cls.__name__ + ".TestCase"
+        StateMachineTestCase.__qualname__ = cls.__qualname__ + ".TestCase"
         return StateMachineTestCase
 
 
 @attr.s()
 class Rule:
     targets = attr.ib()
-    function = attr.ib(repr=attrgetter("__qualname__"))
+    function = attr.ib(repr=get_pretty_function_description)
     arguments = attr.ib()
     preconditions = attr.ib()
     bundles = attr.ib(init=False)
@@ -524,6 +547,13 @@ def _convert_targets(targets, target):
                 )
             raise InvalidArgument(msg % (t, type(t)))
         while isinstance(t, Bundle):
+            if isinstance(t, BundleConsumer):
+                note_deprecation(
+                    f"Using consumes({t.name}) doesn't makes sense in this context.  "
+                    "This will be an error in a future version of Hypothesis.",
+                    since="2021-09-08",
+                    has_codemod=False,
+                )
             t = t.name
         converted_targets.append(t)
     return tuple(converted_targets)
@@ -570,15 +600,15 @@ def rule(
     targets: Sequence[Bundle[Ex]],
     target: None = ...,
     **kwargs: SearchStrategy,
-) -> _RuleWrapper[Ex]:
-    raise NotImplementedError
+) -> _RuleWrapper[Ex]:  # pragma: no cover
+    ...
 
 
 @overload
 def rule(
     *, target: Bundle[Ex], targets: _OmittedArgument = ..., **kwargs: SearchStrategy
-) -> _RuleWrapper[Ex]:
-    raise NotImplementedError
+) -> _RuleWrapper[Ex]:  # pragma: no cover
+    ...
 
 
 @overload
@@ -587,8 +617,8 @@ def rule(
     target: None = ...,
     targets: _OmittedArgument = ...,
     **kwargs: SearchStrategy,
-) -> Callable[[Callable[..., None]], Callable[..., None]]:
-    raise NotImplementedError
+) -> Callable[[Callable[..., None]], Callable[..., None]]:  # pragma: no cover
+    ...
 
 
 def rule(
@@ -657,15 +687,15 @@ def initialize(
     targets: Sequence[Bundle[Ex]],
     target: None = ...,
     **kwargs: SearchStrategy,
-) -> _RuleWrapper[Ex]:
-    raise NotImplementedError
+) -> _RuleWrapper[Ex]:  # pragma: no cover
+    ...
 
 
 @overload
 def initialize(
     *, target: Bundle[Ex], targets: _OmittedArgument = ..., **kwargs: SearchStrategy
-) -> _RuleWrapper[Ex]:
-    raise NotImplementedError
+) -> _RuleWrapper[Ex]:  # pragma: no cover
+    ...
 
 
 @overload
@@ -674,8 +704,8 @@ def initialize(
     target: None = ...,
     targets: _OmittedArgument = ...,
     **kwargs: SearchStrategy,
-) -> Callable[[Callable[..., None]], Callable[..., None]]:
-    raise NotImplementedError
+) -> Callable[[Callable[..., None]], Callable[..., None]]:  # pragma: no cover
+    ...
 
 
 def initialize(
@@ -797,7 +827,7 @@ def precondition(precond: Callable[[Any], bool]) -> Callable[[TestFunc], TestFun
 
 @attr.s()
 class Invariant:
-    function = attr.ib()
+    function = attr.ib(repr=get_pretty_function_description)
     preconditions = attr.ib()
     check_during_init = attr.ib()
 
@@ -857,7 +887,7 @@ LOOP_LABEL = cu.calc_label_from_name("RuleStrategy loop iteration")
 
 class RuleStrategy(SearchStrategy):
     def __init__(self, machine):
-        SearchStrategy.__init__(self)
+        super().__init__()
         self.machine = machine
         self.rules = list(machine.rules())
 
